/*
	Copyright (c) 2000, Tinic Uro <tinic@tinic.net> 
	All rights reserved. Please read the LICENSE file 
	which included in this distribution for details.
*/
#ifndef _ZCOLOR_H_
#define _ZCOLOR_H_

#include "ZSystem.h"

/// helper functions to deal with colorspace conversions
class ZColor
{
	public:
							///
						ZColor() {}

							/// Convert to premultiplied alpha value space
static	inline	uint32	AlphaMultiply(uint32 color)
						{
							uint32 r = (color>>24)*((color>>16)&0xFF);
							uint32 g = (color>>24)*((color>> 8)&0xFF);
							uint32 b = (color>>24)*((color    )&0xFF);
							return	(color&0xFF000000)|
									(((r+(r>>8)+1)>>8)<<16)|
									(((g+(g>>8)+1)>>8)<< 8)|
									(((b+(b>>8)+1)>>8)    );
						}

							/// Convert from premultiplied alpha value space
static	inline	uint32	AlphaDivide(uint32 color)
						{
							uint32 r = (0xFF*((color>>16)&0xFF))/(color>>24);
							uint32 g = (0xFF*((color>> 8)&0xFF))/(color>>24);
							uint32 b = (0xFF*((color    )&0xFF))/(color>>24);
							return	(color&0xFF000000)|(r<<16)|(g<< 8)|(b);
						}

							/// Convert to premultiplied alpha value space
static	inline	void	AlphaMultiply(uint32 *buffer, sint32 count)
						{
							for(count--;count>=0;count--)
							{
								uint32 c = buffer[count];
								uint32 r = (c>>24)*((c>>16)&0xFF);
								uint32 g = (c>>24)*((c>> 8)&0xFF);
								uint32 b = (c>>24)*((c    )&0xFF);
								buffer[count] = 
										(c&0xFF000000)|
										(((r+(r>>8)+1)>>8)<<16)|
										(((g+(g>>8)+1)>>8)<< 8)|
										(((b+(b>>8)+1)>>8)    );
							}
						}

							/// Convert from premultiplied alpha value space
static	inline	void	AlphaDivide(uint32 *buffer, sint32 count)
						{
							for(count--;count>=0;count--)
							{
								uint32 c = buffer[count];
								uint32 r = (0xFF*((c>>16)&0xFF))/(c>>24);
								uint32 g = (0xFF*((c>> 8)&0xFF))/(c>>24);
								uint32 b = (0xFF*((c    )&0xFF))/(c>>24);
								buffer[count] = (c&0xFF000000)|(r<<16)|(g<< 8)|(b);
							}
						}

							/// Interpolate between two colors
static	inline	uint32	Interpolate(uint32 a, uint32 b, uint08 pos)
						{
							// interpolate correctly between two possibly transparent colors 
							// assumes premultiplied color value   
							uint32 ca=((a>>24)     )*(uint32)(0xFF-pos)+((b>>24)     )*(uint32)pos;
							uint32 cr=((a>>16)&0xFF)*(uint32)(0xFF-pos)+((b>>16)&0xFF)*(uint32)pos;
							uint32 cg=((a>> 8)&0xFF)*(uint32)(0xFF-pos)+((b>> 8)&0xFF)*(uint32)pos;
							uint32 cb=((a    )&0xFF)*(uint32)(0xFF-pos)+((b    )&0xFF)*(uint32)pos;
							return	(((ca+(ca>>8)+1)>>8)<<24)|
									(((cr+(cr>>8)+1)>>8)<<16)|
									(((cg+(cg>>8)+1)>>8)<< 8)|
									(((cb+(cb>>8)+1)>>8)    );
						}
};


#endif