/*
	Copyright (c) 2000, Tinic Uro <tinic@tinic.net> 
	All rights reserved. Please read the LICENSE file 
	which included in this distribution for details.
*/
#include "ZLine.h"
#include <math.h>

ZLine::ZLine()
{
}

ZLine::ZLine(ZPoint p0, ZPoint p1)
{
	P0 = p0;
	P1 = p1;
}

ZLine::~ZLine()
{
}

void ZLine::Set(ZPoint p0, ZPoint p1)
{
	P0 = p0;
	P1 = p1;
}

bool ZLine::Intersects(const ZLine &from) const
{
	if( (Orientation(from.P0) < Orientation(from.P1)) &&
		(from.Orientation(P0) < from.Orientation(P1)))
	{
		return true;
	}
	return false;
}

ZPoint ZLine::Intersection(const ZLine &from) const
{
	float32 dy1 = P1.y-P0.y;
	float32 dx1 = P1.x-P0.x;
	float32 dy2 = from.P1.y-from.P0.y;
	float32 dx2 = from.P1.x-from.P0.x;
	float32 e = (dy1 * P0.x) - (dx1 * P0.y);
	float32 f = (dy2 * from.P0.x) - (dx2 * from.P0.y);
	float32 s = ((dx2)*(from.P0.y-P0.y)-(dy2)*(from.P0.x-P0.x))/((dx2)*(dy1)-(dy2)*(dx1));
	float32 t = ((dx1)*(from.P0.y-P0.y)-(dy1)*(from.P0.x-P0.x))/((dx2)*(dy1)-(dy2)*(dx1));
	return ZPoint(
		(-(e * dx2 - dx1 * f)/(-dy1 * dx2 + dy2 * dx1)),
		(-(e * dy2 - dy1 * f)/(-dy1 * dx2 + dy2 * dx1))
	);
}

float32 ZLine::Orientation(const ZPoint &from) const
{
	return (P0.x * P1.y - P0.y * P1.x + P0.y * from.x - P0.x * from.y + P1.x * from.y - from.x * P1.y );
}

ZPoint ZLine::Center() const
{
	return (P0+P1)/2;
}

float32	ZLine::Length() const
{
	ZPoint v(P1-P0); v*=v;
	return (float32)sqrt(v.x + v.y);
}

ZLine ZLine::Head(float32 length) const
{
	ZPoint c(Center());
	ZPoint p(P0-c);
	p.Set(-p.y,p.x);
	p *= length/p.Length();
	return ZLine(P0-p,P0+p);
}

ZLine ZLine::Tail(float32 length) const
{
	ZPoint c(Center());
	ZPoint p(P1-c);
	p.Set(p.y,-p.x);
	p *= length/p.Length();
	return ZLine(P1-p,P1+p);
}
