/*
	Copyright (c) 2000, Tinic Uro <tinic@tinic.net> 
	All rights reserved. Please read the LICENSE file 
	which included in this distribution for details.
*/
#ifndef _Z_PATH_H_
#define _Z_PATH_H_

#include "ZMatrix.h"

/// Data structure represeting a path
class ZPath
{
	public:

			/// This iterator class can be used to retrieve path data
		class Iterator
		{
			public:
									///
								 Iterator() { }
								 
								    /// Interate was called
				virtual	void	 Init() = 0;
									/// Move to action specified
				virtual	void	 MoveTo(ZPoint point) = 0;
									/// Line to action specified
				virtual	void	 LineTo(ZPoint point) = 0;
									/// Quadratic curve action specified
				virtual	void	 CurveTo(ZPoint control, ZPoint point) = 0;
									/// Cubic curve action specified
				virtual	void	 CurveTo(ZPoint control1, ZPoint control2, ZPoint point) = 0;
									/// Arc action specified
				virtual	void	 ArcTo(ZPoint radius, ZPoint point, float32 rotation = 0, sint32 sweep = 1) = 0;
									/// Close action specified
				virtual	void	 Close() = 0;

		};

							///
						 ZPath();
							///
						 ZPath(const ZPath &from);
							///
		virtual			~ZPath();

							/// Set initial cursor position
				void	 AddMoveTo(ZPoint point);
							/// Spawn line from cursor position to point
				void	 AddLineTo(ZPoint point);
							/// Spawn quadratic curve from cursor position to point using control point
				void	 AddCurveTo(ZPoint control, ZPoint point);
							/// Spawn cubic curve from cursor position to point using control points
				void	 AddCurveTo(ZPoint control1, ZPoint control2, ZPoint point);
							/// Spawn arc from cursor position to point
				void	 AddArcTo(ZPoint radius, ZPoint point, float32 rotation = 0, sint32 sweep_arc = 1);
							/// Spawn line from current cursor position to initial cursor position to close the shape
				void	 AddClose();
							/// Clear path data
				void	 MakeEmpty();
							/// Iterate trough complete path data to specified iterator
				void	 Iterate(Iterator *iterator) const;

							/// Write path data to stream and return updated pointer
				void	*ToStream(void *stream);
							/// Read path data from stream and return updated pointer
				void	*FromStream(void *stream);
							/// Return path data size in bytes
				sint32 	 StreamByteSize();

				ZPath	&operator+=(const ZPath &from);
				ZPath	&operator=(const ZPath &from);

	private:

				void	 ExtendIfNeeded(sint32 addsize);

		uint32 			*_data;
		sint32			 _used_size;
		sint32			 _allc_size;
};

#endif
