#ifndef _ZPOINT_H_
#define _ZPOINT_H_

class ZPoint;

#include "ZSystem.h"
#include "ZMatrix.h"
#include <math.h>

/// Data structure representing a point in a 2D space
class ZPoint
{
	public:

			/// x coordinate
		float32	x;
			/// y coordinate
		float32	y;

					///
				ZPoint(float32 _x=0, float32 _y=0):
					x(_x),
					y(_y)
				{
				}

					///
				ZPoint(const ZPoint &from):
					x(from.x),
					y(from.y)
				{
				}
				
					///				
		void	Set(float32 _x, float32 _y)
				{
					x = _x; y = _y;
				}

					///
		float32	Length() const
				{
					return (float32)sqrt(x*x + y*y);
				}

					///
		ZPoint	&operator=(const ZPoint &from)
				{
					x = from.x;
					y = from.y;
					return *this;
				}

					///
		ZPoint	operator+(const ZPoint &from) const
				{
					return ZPoint(x+from.x,y+from.y);
				}

					///
		ZPoint	operator+(float32 v) const
				{
					return ZPoint(x+v,y+v);
				}

					///
		ZPoint	operator-(const ZPoint &from) const
				{
					return ZPoint(x-from.x,y-from.y);
				}

					///
		ZPoint	operator-(float32 v) const
				{
					return ZPoint(x-v,y-v);
				}

					///
		ZPoint	operator*(const ZPoint &from) const
				{
					return ZPoint(x*from.x,y*from.y);
				}

					///
		ZPoint	operator*(float32 v) const
				{
					return ZPoint(x*v,y*v);
				}

					///
		ZPoint	operator/(const ZPoint &from) const
				{
					return ZPoint(x/from.x,y/from.y);
				}

					///
		ZPoint	operator/(float32 v) const
				{
					return ZPoint(x/v,y/v);
				}

					///
		ZPoint	&operator+=(const ZPoint &from)
				{
					x+=from.x;
					y+=from.y;
					return *this;
				}

					///
		ZPoint	&operator+=(float32 v)
				{
					x+=v;
					y+=v;
					return *this;
				}

					///
		ZPoint	&operator-=(const ZPoint &from)
				{
					x-=from.x;
					y-=from.y;
					return *this;
				}

					///
		ZPoint	&operator-=(float32 v)
				{
					x-=v;
					y-=v;
					return *this;
				}

					///
		ZPoint	&operator*=(const ZPoint &from)
				{
					x*=from.x;
					y*=from.y;
					return *this;
				}

					///
		ZPoint	&operator*=(float32 v)
				{
					x*=v;
					y*=v;
					return *this;
				}

					///
		ZPoint	&operator/=(const ZPoint &from)
				{
					x/=from.x;
					y/=from.y;
					return *this;
				}

					///
		ZPoint	&operator/=(float32 v)
				{
					x/=v;
					y/=v;
					return *this;
				}

					///
		bool	operator==(const ZPoint &from) const
				{
					if(x==from.x && y==from.y) return true;
					return false;
				}

					///
		bool	operator!=(const ZPoint &from) const
				{
					if(x!=from.x || y!=from.y) return true;
					return false;
				}

// These two are implemented in ZMatrix.cpp
					///
		ZPoint	operator*(const ZMatrix &from) const;
					///
		ZPoint	&operator*=(const ZMatrix &from);

}///
;

#endif
