#include "ZStencil.h"
#include "ZTools.h"
#include <stdio.h>
#include <math.h>
#include <string.h>

///
ZStencil::ZStencil(void):
	xmin( (1<<30)),
	ymin( (1<<30)),
	xmax(-(1<<30)),
	ymax(-(1<<30))
{
}

///
ZStencil::~ZStencil()
{
}

void ZStencil::SetBounds(float32 _xmin, float32 _ymin, float32 _xmax, float32 _ymax)
{
	xmin = _xmin;
	ymin = _ymin;
	xmax = _xmax;
	ymax = _ymax;

	// Let stencil know that something will happen soon.
	Reset();
}

void ZStencil::SetPathMatrix(const ZMatrix &matrix)
{
	pmtx = matrix;
	pmtx.Inverse();
}

///
void ZLinearGradientStencil::SetPosition(sint32 x, sint32 y)
{
	ZPoint p0((ZPoint((float32)x+0.f,(float32)y)*PathMatrix())*StencilMatrix());
	ZPoint p1((ZPoint((float32)x+1.f,(float32)y)*PathMatrix())*StencilMatrix());

	sint32 ix0=(sint32)(((p0.x-PathMinX())/(PathMaxX()-PathMinX()))*65536);
	sint32 ix1=(sint32)(((p1.x-PathMinX())/(PathMaxX()-PathMinX()))*65536);

	curx = ix0;
	incx = ix1-ix0;
}
                                                                            
///
uint32 ZLinearGradientStencil::GetPixel(bool skip)
{
	if(!skip)
	{
		uint32 color = GetColor(curx>>(16-GRADIENTBUFFERBITS)); 
		curx+=incx;
		return color;
	}
	curx+=incx;
	return 0;
}


///
void ZRadialGradientStencil::SetPosition(sint32 x, sint32 y)
{
	ZPoint p0((ZPoint((float32)x+0.f,(float32)y)*PathMatrix())*StencilMatrix());
	ZPoint p1((ZPoint((float32)x+1.f,(float32)y)*PathMatrix())*StencilMatrix());

	sint32 ix0=(sint32)((((p0.x-PathMinX())/(PathMaxX()-PathMinX()))-.5)*2*65536);
	sint32 ix1=(sint32)((((p1.x-PathMinX())/(PathMaxX()-PathMinX()))-.5)*2*65536);

	curx = ix0;
	incx = ix1-ix0;

	sint32 iy0=(sint32)((((p0.y-PathMinY())/(PathMaxY()-PathMinY()))-.5)*2*65536);
	sint32 iy1=(sint32)((((p1.y-PathMinY())/(PathMaxY()-PathMinY()))-.5)*2*65536);

	cury = iy0;
	incy = iy1-iy0;
}
                                                                            
///
uint32 ZRadialGradientStencil::GetPixel(bool skip)
{
	if(!skip)
	{
		uint32 tmpx = curx>>(16-GRADIENTBUFFERBITS);
		uint32 tmpy = cury>>(16-GRADIENTBUFFERBITS);
		uint32 color = GetColor(integer_sqrt(tmpx*tmpx + tmpy*tmpy)); 
		curx+=incx;
		cury+=incy;
		return color;
	}
	curx+=incx;
	cury+=incy;
	return 0;
}

///
ZBitmapStencil::ZBitmapStencil() 
{ 
	for(sint32 i=0;i<MIMMAPLEVEL;i++) mipb[i]=0;
	bmp = 0;
}

///
ZBitmapStencil::~ZBitmapStencil() 
{ 
	for(sint32 i=0;i<MIMMAPLEVEL;i++) delete [] mipb[i];
}


/**	Set new bitmap pattern to be used for rendering.
	Bitmap should be in ARGB 32 bit mode with premultiplied alpha values. 
	Please note that this method will make a provate copy of the pattern,
	which means that you can free your buffer at any time after this call.
	In fact this is done since this method anyway creates a mip map for the
	pattern, e.g. allocation buffers anyway.
*/
void ZBitmapStencil::SetBitmap(const uint32 *buffer, uint32 width, uint32 height)
{
	for(sint32 i=0;i<MIMMAPLEVEL;i++) 
	{	
		delete [] mipb[i];
		mipb[i]=0;
	}
	if(buffer && width && height)
	{

		mipb[0] = new uint32[width * height];
		mipx[0] = width;
		mipy[0] = height;
		memcpy(mipb[0],buffer,width * height * sizeof(uint32));
		for(sint32 c=1;c<MIMMAPLEVEL;c++)
		{
			mipx[c] = mipx[c-1]/2;
			mipy[c] = mipy[c-1]/2;
			if(mipx[c]==0 || mipy[c]==0)
			{
				sint32 sc=0;
				sint32 sa=0;
				sint32 sr=0;
				sint32 sg=0;
				sint32 sb=0;
				for(uint32 y=0;y<mipy[c-1];y++)
				{
					for(uint32 x=0;x<mipx[c-1];x++)
					{ 
						sc++;
						uint32 p = mipb[c-1][y*mipx[c-1]+x];
						sa += (p>>24)&0xFF;
						sr += (p>>16)&0xFF;
						sg += (p>> 8)&0xFF;
						sb += (p    )&0xFF;
					}
				}
				mipb[c] = new uint32[1];
				mipx[c] = 1;
				mipy[c] = 1;
				mipb[c][0] = 
					((sa/sc)<<24)|
					((sr/sc)<<16)|
					((sg/sc)<< 8)|
					((sb/sc)    );
			}
			else
			{
				mipb[c]  = new uint32[mipx[c]*mipy[c]];
				for(uint32 y=0;y<mipy[c];y++)
				{
					for(uint32 x=0;x<mipx[c];x++)
					{
						uint32 p0 = mipb[c-1][((y*2)+0)*mipx[c-1]+(x*2)+0];
						uint32 p1 = mipb[c-1][((y*2)+0)*mipx[c-1]+(x*2)+1];
						uint32 p2 = mipb[c-1][((y*2)+1)*mipx[c-1]+(x*2)+0];
						uint32 p3 = mipb[c-1][((y*2)+1)*mipx[c-1]+(x*2)+1];
						mipb[c][y*mipx[c]+x] = 
							(((((p0>>24)&0xFF) + ((p1>>24)&0xFF) + ((p2>>24)&0xFF) + ((p3>>24)&0xFF))/4)<<24)|
							(((((p0>>16)&0xFF) + ((p1>>16)&0xFF) + ((p2>>16)&0xFF) + ((p3>>16)&0xFF))/4)<<16)|
							(((((p0>> 8)&0xFF) + ((p1>> 8)&0xFF) + ((p2>> 8)&0xFF) + ((p3>> 8)&0xFF))/4)<< 8)|
							(((((p0    )&0xFF) + ((p1    )&0xFF) + ((p2    )&0xFF) + ((p3    )&0xFF))/4)    );
					}
				}
			}
		}
	}
}

///
void ZBitmapStencil::SetPosition(sint32 x, sint32 y)
{
	ZPoint p0((ZPoint((float32)x+0.f,(float32)y)*PathMatrix())*StencilMatrix());
	ZPoint p1((ZPoint((float32)x+1.f,(float32)y)*PathMatrix())*StencilMatrix());

	sint32 ix0=(sint32)(((p0.x-PathMinX())/(PathMaxX()-PathMinX()))*65536*16);
	sint32 ix1=(sint32)(((p1.x-PathMinX())/(PathMaxX()-PathMinX()))*65536*16);

	curx = ix0;
	incx = ix1-ix0;

	sint32 iy0=(sint32)(((p0.y-PathMinY())/(PathMaxY()-PathMinY()))*65536*16);
	sint32 iy1=(sint32)(((p1.y-PathMinY())/(PathMaxY()-PathMinY()))*65536*16);

	cury = iy0;
	incy = iy1-iy0;

	// compute mip map level
	if(bmp==0)
	{
		sint32 levl=0;
		for(float32 iter=(p1-p0).Length()/2;iter>=1;iter/=2.) levl++;
		levl= ZMIN(MIMMAPLEVEL-1,levl);

		// set correct mip map
		bmp = mipb[levl];
		wdh = mipx[levl];
		hgt = mipy[levl];
	}

	curx *= wdh;
	cury *= hgt;
	incx *= wdh;
	incy *= hgt;
}
                                                                            
///
uint32 ZBitmapStencil::GetPixel(bool skip)
{
	if(!skip)
	{
		if(bmp)
		{
			uint32 cx = (curx>>20)+(wdh<<12);
			uint32 cy = (cury>>20)+(hgt<<12);

			uint32 x0 = ((cx  )%wdh); // Uaaah!!!
			uint32 y0 = ((cy  )%hgt);
			uint32 x1 = ((x0+1)%wdh);
			uint32 y1 = ((y0+1)%hgt);

			uint32 in_pix0 = bmp[y0*wdh+x0];
			uint32 in_pix1 = bmp[y0*wdh+x1];
			uint32 in_pix2 = bmp[y1*wdh+x0];
			uint32 in_pix3 = bmp[y1*wdh+x1];

			if((in_pix0>>24) || (in_pix1>>24) || (in_pix2>>24) || (in_pix3>>24))
			{
				sint32  mx1 = (curx>>8)&0xFFF;
				sint32  mx0 = 4096-mx1;
				sint32  my1 = (cury>>8)&0xFFF;
				sint32  my0 = 4096-my1;

				uint32 m1 = (my0*mx0);
				uint32 m2 = (my0*mx1);
				uint32 m3 = (my1*mx0);
				uint32 m4 = (my1*mx1);

				uint32 a = (m1*((in_pix0>>24)&0xFF) + m2*((in_pix1>>24)&0xFF) + m3*((in_pix2>>24)&0xFF) + m4*((in_pix3>>24)&0xFF))>>24;
				uint32 r = (m1*((in_pix0>>16)&0xFF) + m2*((in_pix1>>16)&0xFF) + m3*((in_pix2>>16)&0xFF) + m4*((in_pix3>>16)&0xFF))>>24;
				uint32 g = (m1*((in_pix0>> 8)&0xFF) + m2*((in_pix1>> 8)&0xFF) + m3*((in_pix2>> 8)&0xFF) + m4*((in_pix3>> 8)&0xFF))>>24;
				uint32 b = (m1*((in_pix0    )&0xFF) + m2*((in_pix1    )&0xFF) + m3*((in_pix2    )&0xFF) + m4*((in_pix3    )&0xFF))>>24;

				curx+=incx;
				cury+=incy;
				return (a<<24) | (r<<16) | (g<< 8) | (b    );
			}
		}
	}
	curx+=incx;
	cury+=incy;
	return 0;
}
