#ifndef _ZSTENCIL_H_
#define _ZSTENCIL_H_

#include "ZMatrix.h"
#include "ZGradient.h"

/// Abtract stencil class used to obtain fill of paths
class ZStencil
{
	public:
							///
						ZStencil(void);
							///
		virtual			~ZStencil();

							/// Set stencil matrix
				void	SetStencilMatrix(const ZMatrix &matrix) { smtx = matrix; smtx.Inverse(); }
							/// Return stencil matrix;
				ZMatrix	&StencilMatrix() { return smtx; }

							/// New path is about to be drawn
		virtual	void	Reset() { };
							/// Set position for new horizontal segment
		virtual	void	SetPosition(sint32 x, sint32 y) = 0;
					  		/// Obtain color at current position
		virtual	uint32	GetPixel(bool skip) = 0;
					  		/// Is this stencil a plain color or depending on x,y?
		virtual	bool	IsMonotone() const = 0;

							/// Minimum untransformed bounds of current path
				float32	PathMinX() const { return xmin; }
							/// Minimum untransformed bounds of current path
				float32	PathMinY() const { return ymin; }
							/// Maximum untransformed bounds of current path
				float32	PathMaxX() const { return xmax; }
							/// Maximum untransformed bounds of current path
				float32	PathMaxY() const { return ymax; }
							/// returns inverse matrix of current path transformation
				const ZMatrix &PathMatrix() const { return pmtx; }

	private:

		friend class ZRenderer;

				void	SetBounds(float32 _xmin, float32 _ymin, float32 _xmax, float32 _ymax);
				void	SetPathMatrix(const ZMatrix &matrix);

				float32	xmin;
				float32	xmax;
				float32	ymin;
				float32	ymax;
				ZMatrix	pmtx;
				ZMatrix smtx;
};

/// Implements plain color stencil
class ZColorStencil : public ZStencil
{
	public:

						ZColorStencil(uint32 c = 0xFFFFFFFF):color(c) {}
		virtual			~ZColorStencil() {}

							/// Set color in ARGB 32 bit format
				void	SetColor(uint32 c) { color = c; }
							/// Obtain color of this color stencil in ARGB 32 bit format
				uint32	Color() { return color; }

		virtual	void	SetPosition(sint32 x, sint32 y) { };
		virtual	uint32	GetPixel(bool skip) { return color; }
							/// returns true
		virtual	bool	IsMonotone() const { return true; }

	private:

				uint32	color;
};

/// Implements linear gradient fill stencil
class ZLinearGradientStencil : public ZStencil, public ZGradient
	{
	public:
							///
						ZLinearGradientStencil() { }
							///
		virtual			~ZLinearGradientStencil() { }
			
		virtual	void	SetPosition(sint32 x, sint32 y);
		virtual	uint32	GetPixel(bool skip);
							/// returns false
		virtual bool	IsMonotone() const { return false; }

	private:
				
				sint32	curx;
				sint32	incx;
};

/// Implements radial gradient fill stencil
class ZRadialGradientStencil : public ZStencil, public ZGradient
{
	public:
							///
						ZRadialGradientStencil() { }
							///
		virtual			~ZRadialGradientStencil() { }
			
		virtual	void	SetPosition(sint32 x, sint32 y);
		virtual	uint32	GetPixel(bool skip);
							/// returns false
		virtual bool	IsMonotone() const { return false; }

	private:
				
				sint32	curx;
				sint32	incx;
				sint32	cury;
				sint32	incy;
};

/// Implements bitmap fill stencil
class ZBitmapStencil : public ZStencil
{
	public:
	
		enum
		{
			MIMMAPLEVEL = 10
		};

						ZBitmapStencil();
		virtual			~ZBitmapStencil();

				void	SetBitmap(const uint32 *buffer, uint32 width, uint32 height);
			
							/// New path is about to be drawn
		virtual	void	Reset() { bmp = 0; };
		virtual	void	SetPosition(sint32 x, sint32 y);
		virtual	uint32	GetPixel(bool skip);
							/// return false
		virtual bool	IsMonotone() const { return false; }

	private:
				uint32	*mipb[MIMMAPLEVEL];
				uint32	mipx[MIMMAPLEVEL];
				uint32	mipy[MIMMAPLEVEL];

				uint32	*bmp;
				uint32	wdh;
				uint32	hgt;
				
				sint32	curx;
				sint32	incx;
				sint32	cury;
				sint32	incy;
};

#endif
