/*
	Copyright (c) 2000, Tinic Uro <tinic@tinic.net> 
	All rights reserved. Please read the LICENSE file 
	which included in this distribution for details.
*/
#include "ptc.h"
#include "ZRenderer.h"
#include "ZOutliner.h"
#include "ZColor.h"

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

int APIENTRY WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow)
{
    try
    {
        // create console
        Console console;
        
        // create format
        Format format(32,0xFF0000,0x00FF00,0x0000FF);
 
        // open the console
        console.open("Zodius",320,240,format);

        // create surface matching console dimensions
        Surface surface(console.width(),console.height(),format);

        ZMatrix matrix;
		float32 rot=0,scl=.1,scl1=0,rad=0;

		ZRenderer renderer;
		ZOutliner outliner;
		DynamicColor color(1.,1.,1.);
//		ZColorStencil stencil(premul_color(0x7FFF0000));
/*		ZColorStencil black(0xFF000000);
		ZRadialGradientStencil stencil;
		stencil.SetColor(ZColor::AlphaMultiply(0x3F00FFFF),0,0.);
		stencil.SetColor(ZColor::AlphaMultiply(0x3F0000FF),1,.5);
		stencil.SetColor(ZColor::AlphaMultiply(0x3F000000),2,1.);*/
		ZBitmapStencil stencil;
		uint32 *bitmap = new uint32[160*120];
		{FILE *fp = fopen("bitmap.raw","rb");
		if(fp)
		{
			sint32 c=0;
			for(;!feof(fp);)
			{
				bitmap[c++] = 0xFF000000 | (fgetc(fp)) | (fgetc(fp)<<8) | (fgetc(fp)<<16);
			}
			ZColor::AlphaMultiply(bitmap,160*120);
			fclose(fp);
			int s=0;
		}}
		stencil.SetBitmap(bitmap,160,120);

		ZMatrix smtx;
		smtx.SetScaling(ZPoint(.1,.1));
		smtx.SetSkewing(ZPoint(.5,0));
		stencil.SetStencilMatrix(smtx);

		int32 cur=-1,cnt=0;
		ZColorStencil *stncs[64];
		ZPath *paths[64][64];

		memset(stncs,0,sizeof(stncs));
		memset(paths,0,sizeof(paths));

		{FILE *fp = fopen("lion.txt","r");
		if(fp)
		{
			char line[1024];
			for(;!feof(fp);)
			{
				fgets(line,1024,fp);
				if(line[0]!='M')
				{
					cur++;
					cnt=0;
					uint32 color = 0;
					sscanf(line,"%x",&color);
					color|=0xFF000000;
					stncs[cur] = new ZColorStencil(color);
				}
				else
				{
					paths[cur][cnt] = new ZPath();
					ZPath path;
					int32 x,y;
					char *scan = line;
					for(;sscanf(scan+1," %d,%d ",&x,&y);)
					{
						if(*scan=='M')
						{
							paths[cur][cnt]->AddMoveTo(ZPoint(x,y)-ZPoint(120,180));
						}
						else
						{
							paths[cur][cnt]->AddLineTo(ZPoint(x,y)-ZPoint(120,180));
						}
						if(*scan == 0) break;
						for(scan++;*scan!='L';scan++) 
						{
							if(*scan == 0) break;
						}
					}
					paths[cur][cnt]->AddClose();
					cnt++;
				}
			}
			fclose(fp);
		}}

        // loop until a key is pressed
		for(;;)
		{
			Key key;
			if (console.key())
			{
				key = console.read();
			}

			// lock surface pixels
			uint32 *pixels = (uint32 *) surface.lock();

			// get surface dimensions
			const int width  = surface.width();
			const int height = surface.height();

			renderer.SetBuffer(pixels,width,height,width);
			renderer.ClearBuffer();

			ZPath path;
			path.AddMoveTo(ZPoint(10,10)	- ZPoint(125,125));
			path.AddLineTo(ZPoint(200,10)	- ZPoint(125,125));
			path.AddLineTo(ZPoint(200,200)	- ZPoint(125,125));
			path.AddLineTo(ZPoint(10,200)	- ZPoint(125,125));
			path.AddClose();

/*			path.AddMoveTo(ZPoint(150,150)	- ZPoint(125,125));
			path.AddLineTo(ZPoint(250,150)	- ZPoint(125,125));
			path.AddLineTo(ZPoint(250,250)	- ZPoint(125,125));
			path.AddLineTo(ZPoint(150,250)	- ZPoint(125,125));
			path.AddClose();

			path.AddMoveTo(ZPoint(20,20)	- ZPoint(125,125));
			path.AddCurveTo(ZPoint(150,20)- ZPoint(125,125),ZPoint(150,150)- ZPoint(125,125));
			path.AddCurveTo(ZPoint(20,150)- ZPoint(125,125),ZPoint(20,20)- ZPoint(125,125));
			path.AddClose();
*/
			switch (key.code())
            {
                case Key::C:	 scl-=.01f; break;
                case Key::V:	 scl+=.01f; break;
                case Key::Z:	 scl-=.25f; break;
                case Key::X:	 scl+=.25f; break;
                case Key::A:	 rad-=.25f; break;
                case Key::S:	 rad+=.25f; break;
                case Key::LEFT:  rot-=.01f; break;
                case Key::RIGHT: rot+=.01f; break;
                case Key::UP:    rot-=.10f; break;
                case Key::DOWN:  rot+=.10f; break;
            }

//			rot+=.10f;
//			scl+=.01f;
//			if(scl>1) scl=0;
//			scl1+=.013f;
//			if(scl1>1) scl1=0;

#if 1
			for(float32 c=0;c<ZPI*2;c+=ZPI*2)
			{
				matrix.SetRotation(rot+c);
				matrix.SetScaling(ZPoint(scl,scl));
				matrix.SetTranslation(ZPoint(160,120));
				renderer.DrawPath(path,matrix,stencil);
			}
//			matrix.SetRotation(rot+1);
//			matrix.SetScaling(ZPoint(scl,scl));
//			matrix.SetTranslation(ZPoint(160,120));
//			renderer.DrawPath(path,matrix,stencil);
#else			
			matrix.SetRotation(rot);
			matrix.SetScaling(ZPoint(scl,scl));
			matrix.SetTranslation(ZPoint(160,120));
			{for(sint32 c=63;c>=0;c--)
			{
				for(sint32 i=63;i>=0;i--)
				{
					if(paths[c][i])
					{
						renderer.DrawPath(*paths[c][i],matrix,*stncs[c]);
					}
				}
			}}
/*			matrix.SetScaling(ZPoint(scl1,scl1));
			matrix.SetTranslation(ZPoint(0+scl*250,80));
			{for(sint32 c=63;c>=0;c--)
			{
				for(sint32 i=63;i>=0;i--)
				{
					if(paths[c][i])
					{
						renderer.DrawPath(*paths[c][i],matrix,*stncs[c]);
					}
				}
			}}*/
#endif
			renderer.FinalizeBuffer(0xFFFFFFFF);

			// unlock surface
			surface.unlock();

			// copy to console
			surface.copy(console);
        
			// update console
			console.update();

			switch (key.code())
			{
				case Key::ESCAPE: return 0;
			}
		}
        // close console
        console.close();
    }
    catch (Error &error)
    {
        // report error
        error.report();
    }

    // exit
    return 0;
}
